/**
 * Copyright (c) 2016-2017 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.samples.rest.edm.dropwizard.server;

import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

import org.glassfish.jersey.server.ServerProperties;
import org.glassfish.jersey.server.filter.RolesAllowedDynamicFeature;
import org.ow2.petals.samples.rest.edm.dropwizard.server.auth.ExampleAuthenticator;
import org.ow2.petals.samples.rest.edm.dropwizard.server.auth.ExampleAuthorizer;
import org.ow2.petals.samples.rest.edm.dropwizard.server.auth.User;
import org.ow2.petals.samples.rest.edm.dropwizard.server.filter.MetricFilter;
import org.ow2.petals.samples.rest.edm.dropwizard.server.repository.Repository;
import org.ow2.petals.samples.rest.edm.dropwizard.server.resources.DocumentMetadataResource;
import org.ow2.petals.samples.rest.edm.dropwizard.server.resources.DocumentResource;
import org.ow2.petals.samples.rest.edm.dropwizard.server.resources.LibraryResource;

import com.codahale.metrics.Counter;
import com.codahale.metrics.JmxReporter;
import com.codahale.metrics.MetricRegistry;

import io.dropwizard.Application;
import io.dropwizard.auth.AuthDynamicFeature;
import io.dropwizard.auth.AuthValueFactoryProvider;
import io.dropwizard.auth.basic.BasicCredentialAuthFilter;
import io.dropwizard.forms.MultiPartBundle;
import io.dropwizard.setup.Bootstrap;
import io.dropwizard.setup.Environment;

public class EDMApplication extends Application<EDMConfiguration> {

    private static final String METRIC_RECEIVE_REQUESTS = "received-requests";

    private final Map<String, Repository> docRepositories = new ConcurrentHashMap<>();

    private final MetricRegistry metrics = new MetricRegistry();

    private final Counter receivedRequests = this.metrics.counter(MetricRegistry.name(METRIC_RECEIVE_REQUESTS));

    private JmxReporter reporter;

    public static void main(final String[] args) throws Exception {
        new EDMApplication().run(args);
    }

    @Override
    public String getName() {
        return "Basic-EDM (Electronic Document Management)";
    }

    @Override
    public void initialize(final Bootstrap<EDMConfiguration> bootstrap) {
        bootstrap.addBundle(new MultiPartBundle());

        this.reporter = JmxReporter.forRegistry(this.metrics).build();

    }

    @Override
    public void run(final EDMConfiguration configuration, final Environment environment) {

        this.reporter.start();

        // Authentication
        environment.jersey()
                .register(new AuthDynamicFeature(
                        new BasicCredentialAuthFilter.Builder<User>().setAuthenticator(new ExampleAuthenticator())
                                .setAuthorizer(new ExampleAuthorizer()).setRealm("HARDCODED-REALM").buildAuthFilter()));
        environment.jersey().register(RolesAllowedDynamicFeature.class);
        // If you want to use @Auth to inject a custom Principal type into your resource
        environment.jersey().register(new AuthValueFactoryProvider.Binder<>(User.class));

        // Metrics
        environment.jersey().register(new MetricFilter(this.receivedRequests));

        // Register resources
        environment.jersey().register(new LibraryResource(this.docRepositories));
        environment.jersey().register(new DocumentResource(this.docRepositories));
        environment.jersey().register(new DocumentMetadataResource(this.docRepositories));

        // Enable WADL generation on http://localhost:8080/application.wadl
        final Map<String, Object> properties = new HashMap<>();
        properties.put(ServerProperties.WADL_FEATURE_DISABLE, false);
        environment.jersey().getResourceConfig().addProperties(properties);
    }

}
