/**
 * Copyright (c) 2016-2017 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.samples.rest.edm.dropwizard.server.resources;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.Map;

import javax.activation.DataHandler;
import javax.activation.DataSource;
import javax.annotation.security.PermitAll;
import javax.annotation.security.RolesAllowed;
import javax.mail.util.ByteArrayDataSource;
import javax.ws.rs.Consumes;
import javax.ws.rs.DELETE;
import javax.ws.rs.GET;
import javax.ws.rs.POST;
import javax.ws.rs.Path;
import javax.ws.rs.PathParam;
import javax.ws.rs.Produces;
import javax.ws.rs.WebApplicationException;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.Response;
import javax.ws.rs.core.Response.Status;
import javax.ws.rs.core.StreamingOutput;

import org.apache.commons.io.IOUtils;
import org.glassfish.jersey.media.multipart.FormDataBodyPart;
import org.glassfish.jersey.media.multipart.FormDataParam;
import org.ow2.petals.samples.rest.edm.dropwizard.server.data.DocumentId;
import org.ow2.petals.samples.rest.edm.dropwizard.server.data.Metadatas;
import org.ow2.petals.samples.rest.edm.dropwizard.server.repository.Repository;

@Path("/library/{library}/document")
@Produces(MediaType.APPLICATION_JSON)
public class DocumentResource {

    private final Map<String, Repository> docRepositories;

    public DocumentResource(final Map<String, Repository> docRepositories) {
        this.docRepositories = docRepositories;
    }

    @POST
    @Consumes({ MediaType.MULTIPART_FORM_DATA })
    @Produces({ MediaType.APPLICATION_JSON })
    @RolesAllowed("ADMIN_DOC")
    public Response create(@PathParam("library") final String library,
            @FormDataParam("file") final InputStream fileInputStream,
            @FormDataParam("file") final FormDataBodyPart fileBody) {
        final Repository docRepository = this.docRepositories.get(library);
        if (docRepository != null) {
            final ByteArrayOutputStream clonedDoc = new ByteArrayOutputStream();
            try {
                IOUtils.copy(fileInputStream, clonedDoc);
            } catch (final IOException e) {
                throw new WebApplicationException(e, Status.INTERNAL_SERVER_ERROR);
            }
            final DataSource documentDs = new ByteArrayDataSource(clonedDoc.toByteArray(),
                    fileBody.getMediaType().toString());
            final DataHandler documentDh = new DataHandler(documentDs);
            final String reference = docRepository.addDocument(documentDh, new Metadatas());
            return Response.ok(new DocumentId(reference)).build();
        } else {
            // The requested library does not exist
            return Response.status(Status.NOT_FOUND).build();
        }
    }

    @Path("/{reference}")
    @GET
    @PermitAll
    public Response get(@PathParam("library") final String library, @PathParam("reference") final String reference) {

        if (LibraryResource.SLOW_LIBRARY.equals(library)) {
            try {
                Thread.sleep(60000);
            } catch (final InterruptedException e) {
                // NOP
            }
        }

        final Repository docRepository = this.docRepositories.get(library);
        if (docRepository != null) {
            final DataHandler documentDh = docRepository.getDocument(reference);
            if (documentDh != null) {
                final StreamingOutput documentStream =  new StreamingOutput() 
                {
                    @Override
                    public void write(final OutputStream output) throws IOException, WebApplicationException 
                    {
                        IOUtils.copy(documentDh.getInputStream(), output);
                        output.flush();
                    }
                };
                return Response
                        .ok(documentStream, MediaType.valueOf(documentDh.getContentType()))
                        .header("content-disposition", "attachment; filename = document-" + reference)
                        .build();
            } else {
                // The requested document does not exist
                return Response.status(Status.NOT_FOUND).build();
            }
        } else {
            // The requested library does not exist
            return Response.status(Status.NOT_FOUND).build();
        }
    }

    @Path("/{reference}")
    @DELETE
    @RolesAllowed("ADMIN_DOC")
    public Response delete(@PathParam("library") final String library, @PathParam("reference") final String reference) {

        final Repository docRepository = this.docRepositories.get(library);
        if (docRepository != null) {
            if (docRepository.delete(reference)) {
                return Response.ok().build();
            } else {
                // The requested document does not exist
                return Response.status(Status.NOT_FOUND).build();
            }
        } else {
            // The requested library does not exist
            return Response.status(Status.NOT_FOUND).build();
        }
    }
}
